/* $Header: /code/doj/readn.c,v 1.4 2008/12/17 20:39:03 doj Exp $ */

#ifdef _MSC_VER
#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#include <io.h>
typedef SSIZE_T ssize_t;
#define read _read
#else
#include <unistd.h>
#endif

/**
   reads from a file descriptor into a buffer. The function finishes
   when either count bytes have been read, the file descriptor reached
   the end-of-file or the file descriptor changed to an error state.

   ripped from Advanced Programming in the Unix Environment by Richard Stevens

   @param fd file descriptor
   @param buf pointer to a buffer
   @param count number of bytes to read into buf

   @return upon success the number of bytes read which may be less than count when end-of-file reached, upon error the negative return code from write() (see the errno variable for details)
 */
ssize_t readn(int fd, void *buf, const size_t count)
{
  /*lint --e{1924}*/

  if(fd<0 || !buf) return -1;

  char *ptr=(char*)buf;
  size_t nleft=count;
  while(nleft > 0)
    {
      ssize_t nread;
      if( (nread=read(fd, ptr, nleft)) < 0)
	return nread;
      else if (nread == 0)
	break;			/* EOF */
      /*lint -e{737} ignore loss of sign*/
      nleft -= nread;
      ptr   += nread;
    }
  /*lint -e{713} ignore loss of precision*/
  return count - nleft;
}
